<?php

declare(strict_types=1);

namespace apexl\Io\modules\user\collections;

use Arrayy\Type\StringCollection;

class permissionCollection extends StringCollection
{
    public static function createFromQueryResult(array $result): permissionCollection
    {
        return self::createFromArray(array_map(fn($result) => $result->permission, $result));
    }
}
